@@ -32,16 +32,20 @@ def singleton(class_):
 
 
 class Worker:
-    def __init__(self, queue_name, metadata_table_name, worker_name, should_wake_up=False):
+    def __init__(self, queue_name, metadata_table_name, worker_name, should_wake_up=False, segment_limit=1000, batch_size=100):
         connection_string = os.environ['AzureWebJobsStorage']
         self.metadata_table_name = metadata_table_name
         self.table_client = TableService(connection_string=connection_string)
         self.queue = QueueClient.from_connection_string(conn_str=connection_string, queue_name=queue_name)
-        self.wake_up_queue = QueueClient.from_connection_string(conn_str=connection_string, queue_name=wake_up_queue_name)
-        self.worker_name = worker_name
+        if should_wake_up:
+            self.wake_up_queue = QueueClient.from_connection_string(conn_str=connection_string, queue_name=wake_up_queue_name)
         self.should_wake_up = should_wake_up
+        self.worker_name = worker_name
         if 'LtbCloudComponentsStorage' in os.environ:
             self.ltb_connection_string = os.environ['LtbCloudComponentsStorage']
+        self.processing_timeout = 120
+        self.segment_limit = segment_limit
+        self.batch_size = batch_size
 
     def trigger_blob(self, blob_name, data=None):
         request_id = self._insert_blob_metadata(blob_name, data)
@@ -102,6 +106,58 @@ class Worker:
             else:
                 raise
 
+    def process_blob(self, request_id, func):
+        guid, request_hash = self._decode_id(request_id)
+        request_entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash)
+        try:
+            blob_name = request_entity['BlobName']
+            if 'Parameters' in request_entity:
+                params = json.loads(request_entity['Parameters'])
+            else:
+                params = None
+
+            offset = 0
+            if 'Offset' in request_entity:
+                offset = request_entity['Offset'].value
+
+            in_blob = BlobClient.from_connection_string(self.ltb_connection_string, container_name=input_container, blob_name=blob_name)
+            result, batch = [], []
+            segment_id = -1
+            whole_blob_read = True
+            for source_segment in self._read_blob_by_segments(in_blob):
+                segment_id += 1
+                if segment_id < offset:
+                    continue
+                if len(batch) == self.batch_size:
+                    result.extend(func(batch, params))
+                    batch = []
+                else:
+                    batch.append(source_segment)
+
+                if segment_id - offset + 1 >= self.segment_limit:
+                    whole_blob_read = False
+                    break
+
+            if len(batch) > 0:
+                result.extend(func(batch, params))
+
+            serialized_result = "\n".join(json.dumps(r) for r in result)
+            out_blob = BlobClient.from_connection_string(self.ltb_connection_string, container_name=output_container, blob_name="%s/%s" % (self.worker_name, blob_name))
+            if offset == 0:
+                out_blob.upload_blob(serialized_result, timeout=600)
+            else:
+                out_blob.append_block("\n" + serialized_result)
+            
+            if not whole_blob_read:
+                request_entity['Offset'] = segment_id + 1
+                self.table_client.update_entity(self.metadata_table_name, request_entity)
+                self._enqueue(request_id)
+                
+        except Exception as e:
+            track = traceback.format_exc()
+            function_failure_entity = {"PartitionKey": self.worker_name, "RowKey": request_id, "FailureMessage": str(e), "StackTrace": track}
+            TableService(connection_string=self.ltb_connection_string).insert_entity(failure_table, function_failure_entity)
+
     def get_result(self, request_id):
         guid, request_hash = self._decode_id(request_id)
         entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash)
@@ -186,6 +242,31 @@ class Worker:
             if source:
                 yield [source]
 
+    @staticmethod
+    def _read_blob_by_segments(input_blob):
+        blob_data = input_blob.download_blob()
+        last_piece = ""
+        for chunk in blob_data.chunks():
+            lines = [l.strip() for l in chunk.decode().split("\r\n") if len(l.strip()) > 0]
+            lines[0] = last_piece + lines[0]
+            last_piece = lines[-1]
+            for l in lines[:-1]:
+                l = unicodedata.normalize("NFKD", l).strip()
+                if len(l) == 0:
+                    continue
+                source_segment = json.loads(l)[0].strip()
+                yield source_segment
+
+        last_piece = unicodedata.normalize("NFKD", last_piece).strip()
+        if len(last_piece) > 0:
+            source = None
+            try:
+                source = json.loads(last_piece)[0].strip()
+            except:
+                pass
+            if source:
+                yield source
+
     @staticmethod
     def _generate_id(data):
         request_id = str(uuid.uuid4()).replace("-", "")
